/***********************************************************************
 ssqls3.cpp - Example showing how to update an SQL row using the
	Specialized SQL Structures feature of MySQL++.

 Copyright (c) 1998 by Kevin Atkinson, (c) 1999-2001 by MySQL AB, and
 (c) 2004-2008 by Educational Technology Resources, Inc.  Others may
 also hold copyrights on code in this file.  See the CREDITS.txt file
 in the top directory of the distribution for details.

 This file is part of MySQL++.

 MySQL++ is free software; you can redistribute it and/or modify it
 under the terms of the GNU Lesser General Public License as published
 by the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 MySQL++ is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with MySQL++; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
 USA
***********************************************************************/

#include "cmdline.h"
#include "printdata.h"
//#define MYSQLPP_SSQLS_NO_STATICS
#include "RenderingMax.h"
#include "StringManager.h"
#include "ConfigureManager.h"

using namespace std;
using namespace CloudResourceManager;


std::string MySqlppHelper::db;
std::string MySqlppHelper::server;
std::string MySqlppHelper::user;
std::string MySqlppHelper::pass;
bool MySqlppHelper::bInitialized = false;
int MySqlppHelper::port = 3306;

MySqlppHelper::MySqlppHelper()
{
    if (!bInitialized)
    {
        bInitialized = true;
        db = ConfigureManager::AppSettings("database");
        server = ConfigureManager::AppSettings("server");
        user = ConfigureManager::AppSettings("user");
        pass = ConfigureManager::AppSettings("pass");
        string strPort = ConfigureManager::AppSettings("port");
        StringManager::string2int(strPort, port);
    }
    con.connect(db.c_str(), server.c_str(), user.c_str(), pass.c_str(), port);
}


void MySqlppHelper::ShowUserCredits_View(const UserCredits_View& credit)
{
    cout<<"CreditId:"<<credit.CreditId<<"UserId:"<<credit.UserId<<"\tUserName:"<<credit.UserName<<"\tLastActivityDate:"<<credit.LastActivityDate
        <<"\tCredits:"<<credit.Credits<<"\tCredits:"<<credit.Credits<<"\tValue:"<<credit.Value<<"\tCurrencyId:"
        <<credit.CurrencyId<<"\tCurrencyName:"<<credit.CurrencyName<<"\tCurrencyAmount:"
        <<credit.CurrencyAmount<<"\tVerified:"<<credit.Verified<<"\tVerifiedOn:"<<credit.VerifiedOn<<endl;
}

void MySqlppHelper::ShowUsers(const Users& user)
{
    cout<<user.equal_list()<<endl;
}

void MySqlppHelper::ShowTasks(const Tasks& task)
{
    cout<<task.equal_list()<<endl;
}

void MySqlppHelper::ShowJobs(const Jobs& job)
{
    cout<<job.equal_list()<<endl;
}

void MySqlppHelper::ShowEC2_Instances(const EC2_Instances& instance)
{
    cout<<instance.equal_list()<<endl;
}

void MySqlppHelper::ShowEC2_Requests(const EC2_Requests& request)
{
    cout<<request.equal_list()<<endl;
}

void MySqlppHelper::ShowUserCredits(const UserCredits& credit)
{
    cout<<credit.equal_list()<<endl;
}


bool MySqlppHelper::UpdateJobStatusId(int jobId, int statusId)
{
    mysqlpp::Query query = con.query();
    query <<"update Jobs set JobStatusId="<< statusId <<" where JobId="<<jobId;
    if (query.exec())
    {
        return query.affected_rows() == 1;
    }
    return false;
}

int MySqlppHelper::SaveNewUsers(const Users& user)
{
    int id = 0;
    mysqlpp::Query query = con.query();
    query<<std::setprecision(16)
        << "insert into Users ("
        << user.field_list(Users_UserName, Users_LastActivityDate,
            Users_Credits, Users_password, Users_email) <<") values("
        <<user.value_list(Users_UserName, Users_LastActivityDate,
            Users_Credits, Users_password, Users_email)
        <<")";
    //cout<<"my insert query:"<<query<<endl;
    FILE_LOG(logDEBUG)<<"my saveNewJob query:"<<query;
    if (query.exec())
    {
        id = query.insert_id();
    }
    return id;
}

bool MySqlppHelper::UpdateUsers(int userId, const Users& user)
{
    mysqlpp::Query query = con.query();
    query<<"update Users set "
        << user.equal_list(Users_UserName, Users_LastActivityDate,
            Users_Credits, Users_password, Users_email)
        <<" where UserId = "<<userId;
    FILE_LOG(logDEBUG)<<"my updateUser query:"<<query;
    if (query.exec())
    {
        if  (query.affected_rows() == 1)
        {
            return true;
        }
    }
    return false;
}

int MySqlppHelper::SaveNewJobs(const Jobs& job)
{
    int id = 0;
    mysqlpp::Query query = con.query();
    query<<std::setprecision(16)
        << "insert into Jobs ("
        << job.field_list(Jobs_TaskId, Jobs_JobStatusId, Jobs_CondorJobId, Jobs_JobTypeId, Jobs_JobOutPutFolder,Jobs_JobInputFolder,
            Jobs_JobError, Jobs_JobInputMainFile, Jobs_JobOutputMainFile,Jobs_JobDesc,Jobs_JobComment,Jobs_JobStartTime, Jobs_JobEndTime,
            Jobs_JobSubmitTime,Jobs_JobRunTime,Jobs_LastCheckTime, Jobs_CreatedBy,Jobs_ModifiedOn,Jobs_JobInputParameter)
        <<") values("
        <<job.value_list(Jobs_TaskId, Jobs_JobStatusId, Jobs_CondorJobId, Jobs_JobTypeId, Jobs_JobOutPutFolder,Jobs_JobInputFolder,
            Jobs_JobError, Jobs_JobInputMainFile, Jobs_JobOutputMainFile,Jobs_JobDesc,Jobs_JobComment,Jobs_JobStartTime, Jobs_JobEndTime,
            Jobs_JobSubmitTime,Jobs_JobRunTime,Jobs_LastCheckTime, Jobs_CreatedBy,Jobs_ModifiedOn,Jobs_JobInputParameter)
        <<")";
    //cout<<"my insert query:"<<query<<endl;
    FILE_LOG(logDEBUG)<<"my saveNewJob query:"<<query;
    if (query.exec())
    {
        id = query.insert_id();
    }
    return id;
}

bool MySqlppHelper::UpdateJobs(int jobId, const Jobs& job)
{
    mysqlpp::Query query = con.query();
    query<<"update Jobs set "
        << job.equal_list(Jobs_TaskId, Jobs_JobStatusId, Jobs_CondorJobId, Jobs_JobTypeId, Jobs_JobOutPutFolder,Jobs_JobInputFolder,
            Jobs_JobError, Jobs_JobInputMainFile, Jobs_JobOutputMainFile,Jobs_JobDesc,Jobs_JobComment,Jobs_JobStartTime, Jobs_JobEndTime,
            Jobs_JobSubmitTime,Jobs_JobRunTime,Jobs_LastCheckTime, Jobs_CreatedBy,Jobs_ModifiedOn,Jobs_JobInputParameter)
        <<" where JobId = "<<jobId;
    FILE_LOG(logDEBUG)<<"my updateJob query:"<<query;
    if (query.exec())
    {
        if  (query.affected_rows() == 1)
        {
            return true;
        }
    }
    return false;
}


int MySqlppHelper::SaveNewTasks(const Tasks& task)
{
    int id = 0;
    mysqlpp::Query query = con.query();
    query<<std::setprecision(16)
        << "insert into Tasks ("
        << task.field_list(Tasks_UserId, Tasks_TaskTypeId, Tasks_TaskStatusId, Tasks_TaskInputParameter, Tasks_TaskInputFolder,
            Tasks_TaskOutputFolder, Tasks_TaskMainFile, Tasks_MaxInstanceNumber, Tasks_SpotInstance, Tasks_EC2InstanceTypeId,
            Tasks_LastCheckTime, Tasks_EC2SnapshotId, Tasks_TaskDesc, Tasks_TaskComment, Tasks_CreatedBy, Tasks_ModifiedOn,
            Tasks_ModifiedBy, Tasks_NotificationTypeId)
        <<") values("
        <<task.value_list(Tasks_UserId, Tasks_TaskTypeId, Tasks_TaskStatusId, Tasks_TaskInputParameter, Tasks_TaskInputFolder,
            Tasks_TaskOutputFolder, Tasks_TaskMainFile, Tasks_MaxInstanceNumber, Tasks_SpotInstance, Tasks_EC2InstanceTypeId,
            Tasks_LastCheckTime, Tasks_EC2SnapshotId, Tasks_TaskDesc, Tasks_TaskComment, Tasks_CreatedBy, Tasks_ModifiedOn,
            Tasks_ModifiedBy, Tasks_NotificationTypeId)
        <<")";
    //cout<<"my insert query:"<<query<<endl;
    FILE_LOG(logDEBUG)<<"my saveNewTask query:"<<query;
    if (query.exec())
    {
        id = query.insert_id();
    }
    return id;
}

bool MySqlppHelper::UpdateTasks(int nTaskId, const Tasks& task)
{
    mysqlpp::Query query = con.query();
    query<<"update Tasks set "
        << task.equal_list(Tasks_UserId, Tasks_TaskTypeId, Tasks_TaskStatusId, Tasks_TaskInputParameter, Tasks_TaskInputFolder,
            Tasks_TaskOutputFolder, Tasks_TaskMainFile, Tasks_MaxInstanceNumber, Tasks_SpotInstance, Tasks_EC2InstanceTypeId,
            Tasks_LastCheckTime,
            Tasks_EC2SnapshotId, Tasks_TaskDesc, Tasks_TaskComment, Tasks_CreatedBy, Tasks_ModifiedOn, Tasks_ModifiedBy)
        <<" where TaskId = "<< nTaskId;
    FILE_LOG(logDEBUG)<<"my UpdateTask query:"<<query;
    if (query.exec())
    {
        if  (query.affected_rows() == 1)
        {
            return true;
        }
    }
    return false;
}

int MySqlppHelper::SaveNewEC2_Instances(const EC2_Instances& instance)
{
    int id = 0;
    mysqlpp::Query query = con.query();
    query<<std::setprecision(16)
        << "insert into EC2_Instances ("
        << instance.field_list(EC2_Instances_StatusId, EC2_Instances_ClusterId, EC2_Instances_TypeId, EC2_Instances_InstanceStartTime,
            EC2_Instances_InstanceEndTime, EC2_Instances_InstanceDesc, EC2_Instances_InstanceComment, EC2_Instances_EC2_InstanceId,
            EC2_Instances_IP, EC2_Instances_LastCheckTime, EC2_Instances_InstancePrice, EC2_Instances_TaskId,
            EC2_Instances_PublicDNS, EC2_Instances_PrivateDNS, EC2_Instances_CreatedBy, EC2_Instances_ModifiedOn,
            EC2_Instances_ModifiedBy)
        <<") values("
        <<instance.value_list(EC2_Instances_StatusId, EC2_Instances_ClusterId, EC2_Instances_TypeId, EC2_Instances_InstanceStartTime,
            EC2_Instances_InstanceEndTime, EC2_Instances_InstanceDesc, EC2_Instances_InstanceComment, EC2_Instances_EC2_InstanceId,
            EC2_Instances_IP, EC2_Instances_LastCheckTime, EC2_Instances_InstancePrice, EC2_Instances_TaskId,
            EC2_Instances_PublicDNS, EC2_Instances_PrivateDNS, EC2_Instances_CreatedBy, EC2_Instances_ModifiedOn,
            EC2_Instances_ModifiedBy)
        <<")";
    //cout<<"my insert query:"<<query<<endl;
    FILE_LOG(logDEBUG)<<"my saveNewEC2_Instance query:"<<query;
    if (query.exec())
    {
        id = query.insert_id();
    }
    return id;
}

bool MySqlppHelper::UpdateEC2_Instances(int instanceId, const EC2_Instances& instance)
{
    mysqlpp::Query query = con.query();
    query<<"update EC2_Instances set "
        << instance.equal_list(EC2_Instances_StatusId, EC2_Instances_ClusterId, EC2_Instances_TypeId, EC2_Instances_InstanceStartTime,
            EC2_Instances_InstanceEndTime, EC2_Instances_InstanceDesc, EC2_Instances_InstanceComment, EC2_Instances_EC2_InstanceId,
            EC2_Instances_IP, EC2_Instances_LastCheckTime, EC2_Instances_InstancePrice, EC2_Instances_TaskId,
            EC2_Instances_PublicDNS, EC2_Instances_PrivateDNS, EC2_Instances_CreatedBy, EC2_Instances_ModifiedOn,
            EC2_Instances_ModifiedBy)
        <<" where InstanceId = "<<instanceId;
    FILE_LOG(logDEBUG)<<"my updateEC2_Instance query:"<<query;
    if (query.exec())
    {
        if  (query.affected_rows() == 1)
        {
            return true;
        }
    }
    return false;
}

int MySqlppHelper::SaveNewEC2_Requests(const EC2_Requests& request)
{
    int id = 0;
    mysqlpp::Query query = con.query();
    query<<std::setprecision(16)
        << "insert into EC2_Requests ("
        << request.field_list(EC2_Requests_RequestTypeId, EC2_Requests_RequestStatusId, EC2_Requests_InstanceId, EC2_Requests_RequestStartTime,
            EC2_Requests_RequestEndTime, EC2_Requests_RequestDesc, EC2_Requests_RequestComment, EC2_Requests_LastCheckTime,
            EC2_Requests_EC2_RequestId, EC2_Requests_TaskId, EC2_Requests_CreatedBy, EC2_Requests_ModifiedOn, EC2_Requests_ModifiedBy)
        <<") values("
        <<request.value_list(EC2_Requests_RequestTypeId, EC2_Requests_RequestStatusId, EC2_Requests_InstanceId, EC2_Requests_RequestStartTime,
            EC2_Requests_RequestEndTime, EC2_Requests_RequestDesc, EC2_Requests_RequestComment, EC2_Requests_LastCheckTime,
            EC2_Requests_EC2_RequestId, EC2_Requests_TaskId, EC2_Requests_CreatedBy, EC2_Requests_ModifiedOn, EC2_Requests_ModifiedBy)
        <<")";
    //cout<<"my insert query:"<<query<<endl;
    FILE_LOG(logDEBUG)<<"my saveNewEC2_Request query:"<<query;
    if (query.exec())
    {
        id = query.insert_id();
    }
    return id;
}

bool MySqlppHelper::UpdateEC2_Requests(int requestId, const EC2_Requests& request)
{
    mysqlpp::Query query = con.query();
    query<<"update EC2_Requests set "
        << request.equal_list(EC2_Requests_RequestTypeId, EC2_Requests_RequestStatusId, EC2_Requests_InstanceId, EC2_Requests_RequestStartTime,
            EC2_Requests_RequestEndTime, EC2_Requests_RequestDesc, EC2_Requests_RequestComment, EC2_Requests_LastCheckTime,
            EC2_Requests_EC2_RequestId, EC2_Requests_TaskId, EC2_Requests_CreatedBy, EC2_Requests_ModifiedOn, EC2_Requests_ModifiedBy)
        <<" where RequestId = "<<requestId;
    FILE_LOG(logDEBUG)<<"my updateEC2_Request query:"<<query;
    if (query.exec())
    {
        if  (query.affected_rows() == 1)
        {
            return true;
        }
    }
    return false;
}

int MySqlppHelper::SaveNewUserCredits(const UserCredits& userCredit)
{
    int id = 0;
    mysqlpp::Query query = con.query();
    query<<std::setprecision(16)
        << "insert into userCredits ("
        << userCredit.field_list(UserCredits_UserId, UserCredits_Value, UserCredits_CurrencyId, UserCredits_CurrencyAmount,
            UserCredits_VerifiedBy, UserCredits_Verified, UserCredits_VerifiedOn)
        <<") values("
        <<userCredit.value_list(UserCredits_UserId, UserCredits_Value, UserCredits_CurrencyId, UserCredits_CurrencyAmount,
            UserCredits_VerifiedBy, UserCredits_Verified, UserCredits_VerifiedOn)
        <<")";
    //cout<<"my insert query:"<<query<<endl;
    FILE_LOG(logDEBUG)<<"my saveNewUserCredit query:"<<query;
    if (query.exec())
    {
        id = query.insert_id();
    }
    return id;
}

bool MySqlppHelper::UpdateUserCredits(int creditId, const UserCredits& userCredit)
{
    mysqlpp::Query query = con.query();
    query<<"update UserCredits set "
        << userCredit.equal_list(UserCredits_UserId, UserCredits_Value, UserCredits_CurrencyId, UserCredits_CurrencyAmount,
            UserCredits_VerifiedBy, UserCredits_Verified, UserCredits_VerifiedOn)
        <<" where CreditId = "<<creditId;
    FILE_LOG(logDEBUG)<<"my updateUserCredit query:"<<query;
    if (query.exec())
    {
        if  (query.affected_rows() == 1)
        {
            return true;
        }
    }
    return false;
}


vector<EC2_Requests> MySqlppHelper::GetNewEC2Requests()
{
    vector<EC2_Requests> result;
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Requests where RequestStatusId=1";
    query.storein(result);
    return result;
}

vector<EC2_Requests> MySqlppHelper::GetOpenEC2Requests()
{
    vector<EC2_Requests> result;
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Requests where RequestStatusId=2";
    query.storein(result);
    return result;
}
vector<EC2_Requests> MySqlppHelper::GetActiveEC2Requests()
{
    vector<EC2_Requests> result;
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Requests where RequestStatusId=3";
    query.storein(result);
    return result;
}

vector<EC2_Instances> MySqlppHelper::GetNewEC2Instances()
{
    vector<EC2_Instances> result;
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Instances where InstanceStatusId=1";
    query.storein(result);
    return result;
}

vector<EC2_Instances> MySqlppHelper::GetOpenEC2Instances()
{
    vector<EC2_Instances> result;
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Instances where InstanceStatusId=3";
    query.storein(result);
    return result;
}


vector<EC2_Instances> MySqlppHelper::GetActiveEC2Instances()
{
    vector<EC2_Instances> result;
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Instances where InstanceStatusId=4";
    query.storein(result);
    return result;
}
vector<EC2_Instances> MySqlppHelper::GetInitiateEC2Instances()
{
    vector<EC2_Instances> result;
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Instances where InstanceStatusId=2";
    query.storein(result);
    return result;
}

vector<EC2_Instances> MySqlppHelper::GetEC2InstancesByUserId(int userId)
{
    vector<EC2_Instances> result;
    mysqlpp::Query query = con.query();
    query <<"select e.InstanceId as InstanceId, e.StatusId as StatusId, e.ClusterId as ClusterId, e.TypeId as TypeId,"
        <<"e.InstanceStartTime as InstanceStartTime, e.InstanceEndTime as InstanceEndTime, e.InstanceDesc as InstanceDesc, "
        <<"e.InstanceComment as InstanceComment, e.EC2_InstanceId as EC2_InstanceId, e.IP as IP, e.LastCheckTime as LastCheckTime,"
        <<"e.InstancePrice as InstancePrice, e.TaskId as TaskId,"
        <<"e.PublicDNS as PublicDNS, e.PrivateDNS as PrivateDNS, e.CreatedOn as CreatedOn, e.CreatedBy as CreatedBy,"
        <<"e.ModifiedOn as ModifiedOn, e.ModifiedBy as ModifiedBy "
        <<" from EC2_Instances e left join Tasks t on e.TaskId = t.TaskId where t.UserId=" << userId;
    query.storein(result);
    return result;
}

vector<Tasks> MySqlppHelper::GetNewTasks()
{
    vector<Tasks> result;
    mysqlpp::Query query = con.query();
    query <<"select * from Tasks where TaskStatusId=1";
    query.storein(result);
    return result;
}

vector<Tasks> MySqlppHelper::GetStartTasks()
{
    vector<Tasks> result;
    mysqlpp::Query query = con.query();
    query <<"select * from Tasks where TaskStatusId=3 or TaskStatusId = 4";
    query.storein(result);
    return result;
}

vector<Jobs> MySqlppHelper::GetIncompleteJobsByTaskId(int taskId)
{
    mysqlpp::Query query = con.query();
    vector<Jobs> result;

    query <<"select * from Jobs where JobStatusId < 5 and TaskId="<< taskId;
    query.storein(result);
    return result;
}

vector<EC2_Instances> MySqlppHelper::GetFinishedEC2Instances()
{
    vector<EC2_Instances> result;
    mysqlpp::Query query = con.query();
    query <<"select e.InstanceId as InstanceId, e.StatusId as StatusId, e.ClusterId as ClusterId, e.TypeId as TypeId,"
        <<"e.InstanceStartTime as InstanceStartTime, e.InstanceEndTime as InstanceEndTime, e.InstanceDesc as InstanceDesc, "
        <<"e.InstanceComment as InstanceComment, e.EC2_InstanceId as EC2_InstanceId, e.IP as IP, e.LastCheckTime as LastCheckTime,"
        <<"e.InstancePrice as InstancePrice, e.TaskId as TaskId,"
        <<"e.PublicDNS as PublicDNS, e.PrivateDNS as PrivateDNS, e.CreatedOn as CreatedOn, e.CreatedBy as CreatedBy,"
        <<"e.ModifiedOn as ModifiedOn, e.ModifiedBy as ModifiedBy "
        <<"from EC2_Instances e left join Tasks t on e.TaskId = t.TaskId "
        <<"where t.TaskStatusId=5";
    query.storein(result);
    return result;
}


vector<Jobs_View> MySqlppHelper::GetInitiateJobs(int taskId)
{
    mysqlpp::Query query = con.query();
    vector<Jobs_View> result;

    query <<"select * from Jobs_View where JobStatusId=2 and TaskId="<< taskId;
    query.storein(result);
    return result;
}

bool MySqlppHelper::GetUsersById(int id, Users& user)
{
    mysqlpp::Query query = con.query();

    query <<"select * from Users where UserId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }

    user = res[0];
    return true;
}

bool MySqlppHelper::GetJobsById(int id, Jobs& job)
{
    mysqlpp::Query query = con.query();
    query <<"select * from Jobs where JobId = " << id;
    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    job = res[0];
    return true;
}

bool MySqlppHelper::GetUserCreditsById(int id, UserCredits& credit)
{
    mysqlpp::Query query = con.query();
    query <<"select * from UserCredits where CreditId = " << id;
    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    credit = res[0];
    return true;
}

bool MySqlppHelper::GetEC2_InstancesById(int id, EC2_Instances& instance)
{
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Instances where InstanceId = " << id;
    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    instance = res[0];
    return true;
}


bool MySqlppHelper::GetTasksById(int id, Tasks& task)
{
    mysqlpp::Query query = con.query();
    query <<"select * from Tasks where TaskId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    /*
    int i;
    for (i = 0; i < res.field_names()->size(); i++)
    {
        // Suppress C++ type name outputs when run under dtest,
        // as they're system-specific.
        cout << res.field_name(i).c_str()<<"\t";
    }
    cout << endl;

    cout<< res.field_types()->size()<<endl;
    for (i = 0; i < res.field_types()->size(); i++)
    {
        // Suppress C++ type name outputs when run under dtest,
        // as they're system-specific.
        cout << (*(res.field_types()))[i].name()<<"\t";
    }
    cout << endl;

    mysqlpp::Row row = res[0];
    cout<<row.size()<<endl;
    for (i = 0; i < row.size(); i ++)
    {
        cout<<row.at(i)<<endl;
    }
    */
    task = res[0];
    return true;
}

bool MySqlppHelper::GetEC2_RequestsById(int id, EC2_Requests& request)
{
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Requests where RequestId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    request = res[0];
    return true;
}

bool MySqlppHelper::GetEC2_InstanceTypesById(int id, EC2_InstanceTypes& type)
{
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_InstanceTypes where TypeId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    type = res[0];
    return true;
}


///////////////////////////////////////
bool MySqlppHelper::GetUserCredits_ViewById(int id, UserCredits_View& credit)
{
    mysqlpp::Query query = con.query();
    query <<"select * from UserCredits_View where CreditId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    credit = res[0];
    return true;
}
bool MySqlppHelper::GetTasks_ViewById(int id, Tasks_View& task)
{
    mysqlpp::Query query = con.query();
    query <<"select * from Tasks_View where TaskId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    task = res[0];
    return true;
}
bool MySqlppHelper::GetEC2_Instances_ViewById(int id, EC2_Instances_View& instance)
{
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Instances_View where InstanceId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    instance = res[0];
    return true;
}
bool MySqlppHelper::GetEC2_Requests_ViewById(int id, EC2_Requests_View& request)
{
    mysqlpp::Query query = con.query();
    query <<"select * from EC2_Requests_View where RequestId = " << id;

    mysqlpp::StoreQueryResult res = query.store();
    if (res.empty())
    {
        return false;
    }
    request = res[0];
    return true;
}


/*

    // Get database access parameters from command line
    const char* db = 0, *server = 0, *user = 0, *pass = "";
    if (!parse_command_line(argc, argv, &db, &server, &user, &pass))
    {
        return 1;
    }

    try
    {
        // Establish the connection to the database server.
        mysqlpp::Connection con(db, server, user, pass);
        mysqlpp::Query query = con.query();
        // Build a query to retrieve the stock item that has Unicode
        // characters encoded in UTF-8 form.
        query <<"select * from stock ";
        query << "where item = " << mysqlpp::quote << "Nuerenberger Bratwurst";

        //print_stock_table(query);
        // Retrieve the row, throwing an exception if it fails.
        mysqlpp::StoreQueryResult res = query.store();
        if (res.empty())
        {
            throw mysqlpp::BadQuery("UTF-8 bratwurst item not found in "
                                    "table, run resetdb");
        }

        // Because there should only be one row in the result set,
        // there's no point in storing the result in an STL container.
        // We can store the first row directly into a stock structure
        // because one of an SSQLS's constructors takes a Row object.
        stock row = res[0];

        // Create a copy so that the replace query knows what the
        // original values are.
        stock orig_row = row;

        // Change the stock object's item to use only 7-bit ASCII, and
        // to deliberately be wider than normal column widths printed
        // by print_stock_table().
        row.item = "Nuerenberger Bratwurst";
        row.num = 4;
        row.weight = 1.45;
        row.price = 5.6;
        row.sdate = mysqlpp::sql_date("2011-03-23");
        row.description = "why it is like this?";

        // Form the query to replace the row in the stock table.
        query.update(orig_row, row);

        // Show the query about to be executed.
        cout << "Query: " << query << endl;

        // Run the query with execute(), since UPDATE doesn't return a
        // result set.
        query.execute();

        // Retrieve and print out the new table contents.
        print_stock_table(query);
    }
    catch (const mysqlpp::BadQuery& er)
    {
        // Handle any query errors
        cerr << "Query error: " << er.what() << endl;
        return -1;
    }
    catch (const mysqlpp::BadConversion& er)
    {
        // Handle bad conversions
        cerr << "Conversion error: " << er.what() << endl <<
        "\tretrieved data size: " << er.retrieved <<
        ", actual size: " << er.actual_size << endl;
        return -1;
    }
    catch (const mysqlpp::Exception& er)
    {
        // Catch-all for any other MySQL++ exceptions
        cerr << "Error: " << er.what() << endl;
        return -1;
    }
    */
