#include "InstanceManager.h"
#include "StringManager.h"
#include "EC2Manager.h"
#include "CondorManager.h"
#define MYSQLPP_SSQLS_NO_STATICS
#include "RenderingMax.h"
#include "ConfigureManager.h"
#include "Blender_RenderManager.h"
#include "CreditManager.h"

using namespace CloudResourceManager;
using namespace std;


int InstanceManager::RequestStatusName2Id(const string& name)
{
    if (name.compare("open") == 0)
    {
        return EC2RequestStatusEnum::Open;
    }
    if (name.compare("active") == 0)
    {
        return EC2RequestStatusEnum::Active;
    }
    if (name.compare("failed") == 0)
    {
        return EC2RequestStatusEnum::Failed;
    }
    if (name.compare("cancelled") == 0)
    {
        return EC2RequestStatusEnum::Canceled;
    }
    if (name.compare("closed") == 0)
    {
        return EC2RequestStatusEnum::Closed;
    }
    return EC2RequestStatusEnum::Error;
}


int InstanceManager::EC2InstanceStatusName2Id(const string& name)
{
    if (name.compare("pending") == 0)
    {
        return EC2InstanceStatusEnum::New;
    }
    if (name.compare("running") == 0)
    {
        return EC2InstanceStatusEnum::Open;
    }

    if (name.compare("shutting-down") == 0 || name.compare("stopping") == 0
        || name.compare("stopped")== 0 || name.compare("terminated") == 0)
    {
        return EC2InstanceStatusEnum::Closed;
    }
    return EC2InstanceStatusEnum::Failed;
}


void InstanceManager::CheckRequestWorker(int requestStatus)
{
    try
    {
        //ConfigLogger.Instance.LogInfo("Check " + instanceStatus.ToString() + "Request");
        FILE_LOG(logINFO)<< "Check requestStatus " << requestStatus;

        vector<EC2_Requests> myEC2Requests;
        switch (requestStatus)
        {
        case EC2RequestStatusEnum::New:
            //get all requests which have "new" status
            myEC2Requests = mySqlppHelper.GetNewEC2Requests();
            break;
        case EC2RequestStatusEnum::Open:
            myEC2Requests = mySqlppHelper.GetOpenEC2Requests();
            break;
        case EC2RequestStatusEnum::Active:
            myEC2Requests = mySqlppHelper.GetActiveEC2Requests();
            break;
        }

        for (int i = 0; i < myEC2Requests.size(); i ++)
        {
            // isn't this stupid of csharp?
            CheckInstanceRequestHelper(myEC2Requests[i]);
        }

        //ConfigLogger.Instance.LogInfo("finish ec2_check_spot_instance_request.ps1");
        FILE_LOG(logINFO)<< "finish ec2_check_spot_instance_request";
    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Check_New_Request:" + e.Message);
        FILE_LOG(logINFO)<< "Instancemanager-Check_New_Request error" << e.what();
    }
}


void InstanceManager::Check_New_Request()
{
    //Check the current status of new EC2 requests
    CheckRequestWorker(EC2RequestStatusEnum::New);

}

        /// <summary>
        /// check the requests which have "open" status.
        /// </summary>
void InstanceManager::Check_Open_Request()
{
    CheckRequestWorker(EC2RequestStatusEnum::Open);
}

        /// <summary>
        /// check the requests which have "Active" status. Get the ec2 instance id from the request
        /// </summary>
void InstanceManager::Check_Active_Request()
{
    CheckRequestWorker(EC2RequestStatusEnum::Active);
}



void InstanceManager::CheckInstanceWorker(int instanceStatus)
{
    int i;
    //ConfigLogger.Instance.LogInfo("Check " + instanceStatus.ToString() + " Instance");
    FILE_LOG(logINFO)<<"Check " << instanceStatus << " Instance";

    try
    {
        vector<EC2_Instances> myInstances;
        switch (instanceStatus)
        {
        case EC2InstanceStatusEnum::New:
            myInstances = mySqlppHelper.GetNewEC2Instances();
            for (i = 0; i < myInstances.size(); i++)
            {
                CheckNewInstanceHelper(myInstances[i]);
            }

            break;
        case EC2InstanceStatusEnum::Open:
            myInstances = mySqlppHelper.GetOpenEC2Instances();
            for (i = 0; i < myInstances.size(); i++)
            {
                CheckOpenInstanceHelper(myInstances[i]);
            }
            break;
        case EC2InstanceStatusEnum::Active:
            myInstances = mySqlppHelper.GetActiveEC2Instances();
            for (i = 0; i < myInstances.size(); i++)
            {
                CheckActiveInstanceHelper(myInstances[i]);
            }
            break;
        case EC2InstanceStatusEnum::Initiate:
            myInstances = mySqlppHelper.GetInitiateEC2Instances();
            for (i = 0; i < myInstances.size(); i++)
            {
                CheckInitiateInstanceHelper(myInstances[i]);
            }
            break;
        }

    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Check New instance:" + e.Message);
        FILE_LOG(logERROR)<<"Instancemanager-Check New instance:" << e.what();
    }

}
        /// <summary>
        /// check the instances which have "new" status. If it is "running", then change the status as "open"
        /// </summary>
void InstanceManager::Check_New_Instance()
{
    //Check the current status of new EC2 instances
    //ConfigLogger.Instance.LogInfo("Check New Instance");
    FILE_LOG(logINFO)<<"Instancemanager-Check New instance:";
    try
    {
        CheckInstanceWorker(EC2InstanceStatusEnum::New);

    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Check New instance:" + e.Message);
        FILE_LOG(logERROR)<<"Instancemanager-Check New instance:" <<e.what();
    }

}

void InstanceManager::Check_Opened_Instance()
{
    FILE_LOG(logINFO)<<"Instancemanager-Check Open instance:";
    try
    {
        CheckInstanceWorker(EC2InstanceStatusEnum::Open);

    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Check New instance:" + e.Message);
        FILE_LOG(logERROR)<<"Instancemanager-Check Open instance:" <<e.what();
    }

}

void InstanceManager::Check_Active_Instance()
{
    //Check the current status of running EC2 instances
    FILE_LOG(logINFO)<<"Instancemanager-Check Active instance:";
    try
    {
        CheckInstanceWorker(EC2InstanceStatusEnum::Active);

    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Check New instance:" + e.Message);
        FILE_LOG(logERROR)<<"Instancemanager-Check Active instance:" <<e.what();
    }
}

void InstanceManager::Check_Initiate_Instance()
{
    FILE_LOG(logINFO)<<"Instancemanager-Check Initiate instance:";
    try
    {
        CheckInstanceWorker(EC2InstanceStatusEnum::Initiate);

    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Check New instance:" + e.Message);
        FILE_LOG(logERROR)<<"Instancemanager-Check Initiate instance:" <<e.what();
    }

}


bool InstanceManager::CreateRegularRequest(int nTaskId, const string& AMIId, const string& strEC2InstanceType)
{
    //create a EC2 regular request to get a EC2 instance
    try
    {
        EC2_Instances myInstance;
        if (EC2Manager::ec2_request_normal_instance(AMIId, strEC2InstanceType, myInstance.EC2_InstanceId))
        {
            myInstance.TaskId = nTaskId;
            myInstance.StatusId = (int)EC2InstanceStatusEnum::New;
            myInstance.InstanceStartTime = mysqlpp::sql_datetime(time(NULL));
            return mySqlppHelper.SaveNewEC2_Instances(myInstance) != 0;
        }
    }

    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Create Regular EC2 Request:" + e.Message);
        FILE_LOG(logINFO)<<"Instancemanager-Create Regular EC2 Request:"<<e.what();
    }
    return false;
}

bool InstanceManager::CreateSpotRequest(int nTaskId, const string& AMIId, const string& ec2InstanceType)
{
    //create a EC2 Spot Request to get a EC2 instance
    try
    {
        //ConfigLogger.Instance.LogInfo("start to create the ec2 spot request: " + AMIId + " type:" + ec2InstanceType);
        FILE_LOG(logINFO)<<"start to create the ec2 spot request: " <<AMIId <<" type:" <<ec2InstanceType;
        string requestId;
        if (!EC2Manager::ec2_request_spot_instance(AMIId, ec2InstanceType, requestId))
        {
            //ConfigLogger.Instance.LogError("CreateSpotRequest does not return value");
            FILE_LOG(logERROR)<<"CreateSpotRequest does not return value";
            return false;
        }
        EC2_Requests myEC2Request;
        myEC2Request.EC2_RequestId = requestId;
        myEC2Request.RequestStatusId = (int)EC2RequestStatusEnum::New;
        myEC2Request.TaskId = nTaskId;
        myEC2Request.RequestStartTime = mysqlpp::sql_datetime(time(NULL));


        if (mySqlppHelper.SaveNewEC2_Requests(myEC2Request)!= 0)
        {
            FILE_LOG(logINFO)<<"new ec2_requests of spot instance is success";
            return true;
        }
        else
        {
            FILE_LOG(logERROR)<<"Serious database error of failure saving new ec2_requests";
            return false;
        }
    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("Instancemanager-Create Spot EC2 Request:" + e.Message);
        FILE_LOG(logERROR)<<"Instancemanager-Create Spot EC2 Request:" << e.what();
    }
}

bool InstanceManager::TerminateEC2Instance(const string& instanceId)
{
    //turn off the ec2 instance
    string toTerminate = ConfigureManager::AppSettings("terminateInstance");
    if (toTerminate.compare("true") == 0)
    {
        return EC2Manager::ec2_destroy_instance(instanceId);
    }
    return true;
}


void InstanceManager::Check_Unused_Instance()
{
    vector<EC2_Instances> myInstances = mySqlppHelper.GetFinishedEC2Instances();
    for (int i = 0; i < myInstances.size(); i ++)
    {
        if (TerminateEC2Instance(myInstances[i].EC2_InstanceId))
        {
            myInstances[i].StatusId = (int)EC2InstanceStatusEnum::Closed;
            myInstances[i].InstanceEndTime = mysqlpp::sql_datetime(time(NULL));
            //myEC2InstanceRepository.Save();
            mySqlppHelper.UpdateEC2_Instances(myInstances[i].InstanceId, myInstances[i]);
        }

    }
}

bool InstanceManager::CreateEC2InstanceRequestHelper(int iTaskId)
{
    Tasks_View myTask;
    if (!mySqlppHelper.GetTasks_ViewById(iTaskId, myTask))
    {
        return false;
    }

    int requestCount = myTask.MaxInstanceNumber;

    string ec2InstanceTypeName = myTask.EC2TypeName;
    string strPlatform = myTask.PlatformName;
    string strAMI = myTask.EC2AMIId;

    bool bUseSpotInstance = myTask.SpotInstance;
    if (requestCount <= 0)
    {
        FILE_LOG(logDEBUG)<<"CreateEC2InstanceRequestHelper: request instance number cannot be less than 1!";
        return false;
    }
    //ConfigLogger.Instance.LogDebug("CreateEC2InstanceRequestHelper: taskid:" + taskId + " strPlatform:" + strPlatform + " ec2InstanceTypeName:" + ec2InstanceTypeName + " ami:" + strAMI + " bUseSpotInstance:" + bUseSpotInstance.ToString() + " requestCount:" + requestCount);
    FILE_LOG(logDEBUG)<<"CreateEC2InstanceRequestHelper: taskid:"<<iTaskId<<" strPlatform:"<<strPlatform<<" ec2InstanceTypeName:"
        <<ec2InstanceTypeName <<" ami:"<< strAMI <<" bUseSpotInstance:"<< bUseSpotInstance << " requestCount:" << requestCount;
    //string currentInstanceType = ConfigurationManager.AppSettings["UserSpotInstance"];
    int i=0;
    while (i < requestCount)
    {

        //ConfigLogger.Instance.LogInfo("start to submit a ec2 request for task:" + taskId);
        FILE_LOG(logINFO)<<"start to submit a ec2 request for task:" << iTaskId;
        bool result = false;

        if (bUseSpotInstance)
        {
            result = CreateSpotRequest(iTaskId, strAMI, ec2InstanceTypeName);
        }
        else
        {
            result = CreateRegularRequest(iTaskId, strAMI, ec2InstanceTypeName);
        }
        //  string requestid=CreateSpotRequest(taskId.ToString());
        if (result)
        {
            //ConfigLogger.Instance.LogInfo("created a ec2 request:" + requestid);
            FILE_LOG(logINFO)<<"created a ec2 request success for taskid:"<<iTaskId<<" strPlatform:"<<strPlatform<<" ec2InstanceTypeName:"
                <<ec2InstanceTypeName <<" ami:"<< strAMI <<" bUseSpotInstance:"<< bUseSpotInstance;
            i++;
        }
        else
        {
            //ConfigLogger.Instance.LogError("create instance request failed for Task: " + taskId);
            FILE_LOG(logERROR)<<"create instance request failed for taskid:"<<iTaskId<<" strPlatform:"<<strPlatform<<" ec2InstanceTypeName:"
                <<ec2InstanceTypeName <<" ami:"<< strAMI <<" bUseSpotInstance:"<< bUseSpotInstance;
            //break;
        }
    }
    return (i == requestCount);
}

void InstanceManager::ScaleInstancesByTask(int taskId )
{
    //ConfigLogger.Instance.LogInfo("start to scale the number of instance");
    FILE_LOG(logINFO)<<"start to scale the number of instance";
    // here we check if EC2 instance is already
    CreateEC2InstanceRequestHelper(taskId);
}

void InstanceManager::CheckInstanceRequestHelper(EC2_Requests& myEC2Request)
{
    //ConfigLogger.Instance.LogInfo("ec2_check_spot_instance_request.ps1");
    try
    {
        string status;
        string strEC2RequestId = myEC2Request.EC2_RequestId;
        int iStatus;
        if (EC2Manager::ec2_check_spot_instance_request(strEC2RequestId, status))
        {
            iStatus = RequestStatusName2Id(status);
            // the logic is like this: we only care about Open and active, other cases will be
            // filtered out by the caller of this function.
            switch (myEC2Request.RequestStatusId)
            {
            case EC2RequestStatusEnum::Open:
                if (iStatus != EC2RequestStatusEnum::Open)
                {
                    myEC2Request.RequestStatusId = iStatus;
                    mySqlppHelper.UpdateEC2_Requests(myEC2Request.RequestId, myEC2Request);
                }
                break;
            case EC2RequestStatusEnum::Active:
                if (myEC2Request.EC2_RequestId.size() == 0)
                {
                    if (EC2Manager::ec2_get_instance_id(strEC2RequestId, myEC2Request.EC2_RequestId))
                    {
                        myEC2Request.RequestStatusId = (int)EC2RequestStatusEnum::Closed;
                        mySqlppHelper.UpdateEC2_Requests(myEC2Request.RequestId, myEC2Request);
                    }
                }
                else
                {
                    // this is possible as database saving may failed last time...???
                    myEC2Request.RequestStatusId = (int)EC2RequestStatusEnum::Closed;
                    mySqlppHelper.UpdateEC2_Requests(myEC2Request.RequestId, myEC2Request);
                }
                break;
            default:
                myEC2Request.RequestStatusId = iStatus;
                mySqlppHelper.UpdateEC2_Requests(myEC2Request.RequestId, myEC2Request);
                break;
            }
        }

        //ConfigLogger.Instance.LogInfo("find a ec2 request:" + myEC2Request.EC2_RequestId + " Status:" + requestResult);
    }
    catch (runtime_error e)
    {
        //ConfigLogger.Instance.LogError("CheckInstanceRequestHelper:" + e.Message);
        FILE_LOG(logERROR)<<"CheckInstanceRequestHelper:" << e.what();
    }
}

        // goal: find new status other than "New" and update
void InstanceManager::CheckNewInstanceHelper(EC2_Instances& myInstance)
{
    //ConfigLogger.Instance.LogDebug("CheckNewInstanceHelper begins.");
    string status;
    if (EC2Manager::ec2_check_instance_status(myInstance.EC2_InstanceId, status))
    {
        int iStatus = EC2InstanceStatusName2Id(status);
        if (iStatus != EC2InstanceStatusEnum::New)
        {
            myInstance.StatusId = iStatus;
            mySqlppHelper.UpdateEC2_Instances(myInstance.InstanceId, myInstance);
        }
    }
    else
    {
        FILE_LOG(logERROR)<<"ec2_check instance status failed";
    }
}


void InstanceManager::CheckOpenInstanceHelper(EC2_Instances& myInstance)
{
    bool bNeedUpdate = false;
    FILE_LOG(logINFO)<<"CheckOpenInstanceHelper begins.";

    // get ip
    if (myInstance.IP.size() == 0)
    {
        if (!EC2Manager::ec2_get_instance_ip(myInstance.EC2_InstanceId, myInstance.IP))
        {
            FILE_LOG(logERROR)<<"instance is bad as we cannot get IP let us update its status";
            string status;
            // it is possible ec2-api failed, lets double check status of instance!!!
            if (EC2Manager::ec2_check_instance_status(myInstance.EC2_InstanceId, status))
            {
                int iStatus = EC2InstanceStatusName2Id(status);
                if (iStatus != EC2InstanceStatusEnum::Open)
                {
                    myInstance.StatusId = iStatus;
                    bNeedUpdate = true;
                }
            }
        }
        else
        {
            bNeedUpdate = true;
        }
    }
    if (myInstance.IP.size() > 0)
    {
        if (CondorManager::check_active_instance(myInstance.IP))
        {
            myInstance.StatusId = (int)EC2InstanceStatusEnum::Active;
            bNeedUpdate = true;
            //ConfigLogger.Instance.LogInfo("Active Instance" + myInstance.EC2_InstanceId + " has ip of " + myInstance.IP);
            FILE_LOG(logINFO)<<"Active Instance" <<myInstance.EC2_InstanceId <<" has ip of " << myInstance.IP;
        }
    }
    else
    {
        // this is a new feature and we need to test
        //ConfigLogger.Instance.LogWarning("ec2 instance maybe bad, or maybe we need wait longer for  " + myInstance.EC2_InstanceId + " to be registered in condor.");
        FILE_LOG(logWARNING)<<"ec2 instance maybe bad, or maybe we need wait longer for  "<< myInstance.EC2_InstanceId <<" to be registered in condor.";

    }
    if (bNeedUpdate)
    {
        mySqlppHelper.UpdateEC2_Instances(myInstance.InstanceId, myInstance);
    }
}


void InstanceManager::CheckActiveInstanceHelper(EC2_Instances& myInstance)
{
    //1.assign the taskid
    //ConfigLogger.Instance.LogInfo("Assign the taskid" + myInstance.TaskId.ToString());
    string strTaskId = StringManager::int2string(myInstance.TaskId);
    if (CondorManager::config_taskId(myInstance.IP, strTaskId))
    {
        string taskStatus = StringManager::int2string(TaskStatusEnum::Initiate);
        if (CondorManager::config_taskStatus(myInstance.IP, taskStatus))
        {
            myInstance.StatusId = (int)EC2InstanceStatusEnum::Initiate;
            //ConfigLogger.Instance.LogInfo("Initiate Instance");
            mySqlppHelper.UpdateEC2_Instances(myInstance.InstanceId, myInstance);
        }
    }
}


void InstanceManager::CheckInitiateInstanceHelper(EC2_Instances& myInstance)
{
    Blender_RenderManager myRenderManager;
    if (myRenderManager.InitiateInstanceByTask((int)myInstance.TaskId))
    {
        myInstance.StatusId = (int) EC2InstanceStatusEnum::Ready;
        mySqlppHelper.UpdateEC2_Instances(myInstance.InstanceId, myInstance);
    }
}

void InstanceManager::Check_BlackListUser()
{
    for (IntSet::const_iterator it= CreditManager::userBlackList.begin(); it != CreditManager::userBlackList.end(); it ++)
    {
        vector<EC2_Instances> instanceList = mySqlppHelper.GetEC2InstancesByUserId(*it);
        for (int i = 0; i < instanceList.size(); i ++)
        {
            string strInstanceId = StringManager::int2string(instanceList[i].InstanceId);
            TerminateEC2Instance(strInstanceId);
        }
    }
}

        /// <summary>
        /// check the status of all instances
        /// </summary>
void InstanceManager::Checkstatus()
{
    Check_New_Request();
    Check_Open_Request();
    Check_Active_Request();
    Check_New_Instance();
    Check_Opened_Instance();
    Check_Active_Instance();
    Check_Initiate_Instance();
    Check_Unused_Instance();
    Check_BlackListUser();
}

