#include <Wt/WStackedWidget>
#include <Wt/WPanel>
#include <Wt/WScrollArea>
#include "MainMenuWidget.h"

namespace Wt
{
    class WStackedWidget;
    class WPanel;
    class WScrollArea;
}

MainMenuWidget::MainMenuWidget(WContainerWidget* parent)
    :WCompositeWidget(parent), m_width(800), m_height(600), m_title("main menu")
{
    m_layout = new WTable();
    setImplementation(m_layout);

    WStackedWidget* contentStack =  new WStackedWidget();
    WScrollArea* scrollArea = new WScrollArea();
    m_menu  = new WMenu(contentStack, Horizontal);
    m_panel = new WPanel();
    doResize();
    m_panel->setCollapsible(true);
    m_panel->setTitleBar(true);
    m_panel->setTitle(m_title);

    m_panel->setCentralWidget(scrollArea);
    scrollArea->setWidget(contentStack);

    m_layout->elementAt(0, 0)->addWidget(m_menu);
    m_layout->elementAt(1, 0)->addWidget(m_panel);
    setStyleClass("MainMenu");
    m_menu->itemSelected().connect(this, &MainMenuWidget::menuItemSelected);
}

void MainMenuWidget::menuItemSelected(WMenuItem* menuItem)
{
    setTitle(menuItem->text().toUTF8());
}

bool MainMenuWidget::selectByName(const std::string& strName)
{
    WMenuItem* item = findItemByName(strName);

    if (item != NULL)
    {
        m_menu->select(item);
        return true;
    }
    return false;
}

WMenuItem* MainMenuWidget::findItemByName(const std::string& strName)
{
    for (int i = 0; i < m_menu->items().size(); i ++)
    {
        WMenuItem* item = m_menu->items()[i];
        if (item->text().toUTF8().compare(strName) == 0)
        {
            return item;
        }
    }
    return NULL;
}

bool MainMenuWidget::setHiddenByName(const std::string& strName, bool bHidden)
{
    WMenuItem* item = findItemByName(strName);

    if (item != NULL)
    {
        m_menu->setItemHidden(item, bHidden);
        return true;
    }
    return false;
}

bool MainMenuWidget::setDisabledByName(const std::string& strName, bool bDisabled)
{
    WMenuItem* item = findItemByName(strName);

    if (item != NULL)
    {
        m_menu->setItemDisabled(item, bDisabled);
        return true;
    }
    return false;
}


void MainMenuWidget::addItem(const WString& strName, WWidget* widget)
{
    m_menu->addItem(strName, widget);
    setTitle(m_menu->currentItem()->text().toUTF8());
}

void MainMenuWidget::setWidth(size_t width)
{
    if (width != m_width)
    {
        m_width = width;
        doResize();
    }
}

void MainMenuWidget::setHeight(size_t height)
{
    if (height != m_height)
    {
        m_height = height;
        doResize();
    }
}

void MainMenuWidget::setTitle(const std::string& strTitle)
{
    if (m_title.compare(strTitle) != 0)
    {
        m_title = strTitle;
        m_panel->setTitle(m_title);
    }
}

void MainMenuWidget::doResize()
{
    m_panel->resize(m_width, m_height);
    m_layout->elementAt(0, 0)->resize(m_width, 50);
}

