#include <Wt/WBreak>
#include <Wt/WCssDecorationStyle>
#include <Wt/WPushButton>
#include <Wt/WTable>
#include <Wt/WTableCell>
#include <Wt/WRegExpValidator>

#include "RegisterWidget.h"

char* RegisterWidget::UserMsg = "Must start with a character, length must be between 4 and 12, "
        "can only include character and digits, case sensitive";
char* RegisterWidget::PasswordMsg = "Must contain at least one character in lower case"
        " and one in upper case and one digit, length must be at least 8"
        " can NOT include other special characters";
char* RegisterWidget::RepeatPasswordMsg = "Must be exactly same as your input password";

char* RegisterWidget::EmailMsg = "Must enter a valid email to receive activation message!";

RegisterWidget::RegisterWidget(WContainerWidget *parent)
    :WContainerWidget(parent), m_validMethod(NULL), m_registerObj(NULL),
    m_notifyMethod(NULL), m_notifyObj(NULL)
{
    setPadding(100, Left | Right);

    IntroText = new WText("<p>Enter your user name and password below.</p>", this);

    WTable *layout = new WTable(this);
    WLabel *usernameLabel = new WLabel("User name: ", layout->elementAt(0, 0));
    layout->elementAt(0, 0)->resize(WLength(14, WLength::FontEx), WLength::Auto);
    Username = new WLineEdit(layout->elementAt(0, 1));
    usernameLabel->setBuddy(Username);
    UserText = new WText(UserMsg, layout->elementAt(0, 2));

    WLabel *passwordLabel = new WLabel("Password: ", layout->elementAt(1, 0));

    Password = new MD5PasswordEdit(layout->elementAt(1, 1));
    passwordLabel->setBuddy(Password);

    PasswordText = new WText(PasswordMsg, layout->elementAt(1, 2));

    WLabel *repeatPasswordLabel = new WLabel("RepeatPassword: ", layout->elementAt(2, 0));

    RepeatPassword = new MD5PasswordEdit(layout->elementAt(2, 1));
    repeatPasswordLabel->setBuddy(RepeatPassword);

    RepeatPasswordText = new WText(RepeatPasswordMsg, layout->elementAt(2, 2));

    // email
    new WBreak(this);
    WLabel *emailLabel = new WLabel("Email: ", layout->elementAt(3, 0));
    layout->elementAt(4, 0)->resize(WLength(14, WLength::FontEx), WLength::Auto);
    Email = new WLineEdit(layout->elementAt(4, 1));
    emailLabel->setBuddy(Email);
    EmailText = new WText(EmailMsg, layout->elementAt(4, 2));


    new WBreak(this);
    RegisterButton = new WPushButton("Register", layout->elementAt(4, 0));
    ///////////////////////////////////////////////////////////
    // validators...
    // user name must start with alphabetics and must be at least 4 char, containing digit or alpha
    WRegExpValidator* userRegExp = new WRegExpValidator(this);
    userRegExp->setRegExp("[a-zA-Z][a-zA-Z0-9]{3,11}");
    userRegExp->setMandatory(true);
    Username->setValidator(userRegExp);

    // password must contain at least one lower case, upper case, digit.
    WRegExpValidator* passRegExp = new WRegExpValidator(this);
    passRegExp->setRegExp("^(?=.{8,})(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z])[a-zA-Z0-9]*$");
    passRegExp->setMandatory(true);
    Password->setValidator(passRegExp);
    ///////////////////

    WRegExpValidator* emailRegExp = new WRegExpValidator(this);
    emailRegExp->setRegExp(
        "([a-z0-9A-Z])"
        "(("
        "([\._-]?)"
        "([a-zA-Z0-9]+)"
        ")*)"
        "[@]"
        "([A-Za-z0-9])"
        "(("
        "([_\.-]?)"
        "([a-z0-9A-Z]+)"
        ")*)"
        "[\.]"
        "(com|edu|net|org|info|tv|([A-Za-z]{2,4}))"
        );
    emailRegExp->setMandatory(true);
    Email->setValidator(emailRegExp);


    Username->keyWentUp().connect(this, &RegisterWidget::checkUser);
    Password->keyWentUp().connect(this, &RegisterWidget::checkPass);
    RepeatPassword->keyWentUp().connect(this, &RegisterWidget::checkRepeat);

    Password->setIncomingSignal(RegisterButton->clicked());
    Password->setOutgoingSignal((MD5PasswordEdit::WObjectMethod)(&RegisterWidget::checkCredentials), this);
    bValidRepeat = true;
}

void RegisterWidget::checkRepeat()
{
    //std::string msg;
    if (Password->text() == RepeatPassword->text())
    {
        //msg += "repeat match!";
        if (!bValidRepeat)
        {
            //msg += "change status from invalid to valid!";
            RepeatPassword->removeStyleClass("Wt-invalid", true);
            bValidRepeat = true;
        }
    }
    else
    {
        //msg += "repeat NOT match!";
        if (bValidRepeat)
        {
            //msg += "change status from valid to invalid!";
            RepeatPassword->addStyleClass("Wt-invalid", true);
            bValidRepeat = false;
        }
    }
    //RepeatPasswordText->setText(msg);
}

void RegisterWidget::checkUser()
{
    Username->validate();
}

void RegisterWidget::checkPass()
{
    Password->validate();
}

void RegisterWidget::setupValidRegisterMethod(ValidRegisterMethod validRegisterMethod, WObject* registerObj)
{
    m_validMethod = validRegisterMethod;
    m_registerObj = registerObj;
}

void RegisterWidget::setupNotifyEventMethod(NotifyMethod notifyMethod, WObject* notifyObj)
{
    m_notifyMethod = notifyMethod;
    m_notifyObj = notifyObj;
}

bool RegisterWidget::checkRegister()
{
    if (Username->validate() != WValidator::Valid)
    {
        IntroText->setText("<p>You entered an invalid user name, it must start with a charactor and can contain"
            " only charactors or digits. Please try again.</p>");
        IntroText->decorationStyle().setForegroundColor(Wt::red);
        Username->setText("");
        return false;
    }

    if (Password->validate() != WValidator::Valid)
    {
        IntroText->setText("<p>You entered an invalid password, it must start contain at least one lower case, one "
            " upper case charactor and one digit. The length must be at least 8. Please try again</p>");
        IntroText->decorationStyle().setForegroundColor(Wt::red);
        Password->setText("");
        RepeatPassword->setText("");
        bValidRepeat = true;
        return false;
    }

    if (!(RepeatPassword->text() == Password->text()))
    {
        IntroText->setText("<p>Your repeat password is not same as your password, please try again.</p>");
        //IntroText->setText("<p>Your repeat password " + RepeatPassword->text() +
        //    " is not same as your password " + Password->text() + ", please try again.</p>");
        IntroText->decorationStyle().setForegroundColor(Wt::red);
        RepeatPassword->setText("");
        bValidRepeat = true;
        return false;
    }
    return true;
}

void RegisterWidget::getUserInfo(std::string& userName, std::string& userPass, std::string& userMail)
{
    userName = Username->text().toUTF8();
    Password->getMD5HexString(userPass);
    userMail = Email->text().toUTF8();
}


void RegisterWidget::checkCredentials(std::string passwordInMD5)
{
    if (checkRegister())
    {
        IntroText->setText("Your user name, password are valid and are ready to be registered in database. Please wait...");
        IntroText->decorationStyle().setForegroundColor(Wt::blue);
        if (m_validMethod)
        {
            bool bValid = boost::bind<bool>(m_validMethod, m_registerObj, _1, _2, _3)(
                Username->text().toUTF8(),
                passwordInMD5,
                Email->text().toUTF8()
                );
            if (!bValid)
            {
                IntroText->setText("<p>You entered the wrong password, or the username "
                          "combination is already in use. If you are a returning "
                          "user, please try to login. If you are a new user, please "
                          "try a different name.</p>");
                IntroText->decorationStyle().setForegroundColor(Wt::red);
                Username->setText("");
                if (m_notifyMethod)
                {
                    boost::bind<void>(m_notifyMethod, m_notifyObj, _1)(RegisterFailed);
                }
            }
            else
            {
                IntroText->setText("Congratulations! Your registration is successful!");
                if (m_notifyMethod)
                {
                    boost::bind<void>(m_notifyMethod, m_notifyObj, _1)(RegisterSuccessful);
                }
            }
        }
    }
}
