#ifndef SESSIONMANAGERWIDGET_H_INCLUDED
#define SESSIONMANAGERWIDGET_H_INCLUDED

#include <Wt/WApplication>
#include <Wt/WBreak>
#include <Wt/WContainerWidget>
#include <Wt/WLineEdit>
#include <Wt/WPushButton>
#include <Wt/WText>
#include <Wt/WLabel>
#include <Wt/WMessageBox>
#include <Wt/WMessageBox>
#include "LoginWidget.h"
#include "RegisterWidget.h"
#include "SubmitWidget.h"
#include <boost/regex.hpp>
#include "DBEnumComboBox.h"
#include <Wt/FileTreeList.h>
#include "DBTableWidget.h"
#include "BasicControls.h"
#include "MainMenuWidget.h"
#include "UploadFileWidget.h"
#include "TaskViewWidget.h"
#include "NotifyEvent.h"
#include <map>
#include <string>


using namespace Wt;

class SessionManagerWidget: public WApplication
{
public:
    SessionManagerWidget(const WEnvironment& env)
        : WApplication(env), m_bLogin(false)
    {
        WApplication::instance()->useStyleSheet("icons/wt.css");

        Initialize();

    }
    void EventListener(NotifyEvent event);
    int VerifyUserPassword(const std::string& userName, const std::string& password);
    bool RegisterUserPassword(const std::string& userName, const std::string& password, const std::string& email);
private:
    bool                m_bLogin;
    std::string         m_myPath;
    MainMenuWidget     *m_mainMenu;
    LoginWidget        *m_login;
    RegisterWidget     *m_register;
    SubmitWidget       *m_submit;
    UploadFileWidget   *m_upload;
    TaskViewWidget     *m_monitor;
    std::string         m_userName;
    std::string         m_userPass;
    std::string         m_userMail;
    std::string         m_userUploadFolder;
    int                 m_userId;


    enum InternalPathEnum
    {
        LOGIN,
        REGISTER,
        UPLOAD,
        SUBMIT,
        MONITOR,
        UNKNOWN,
    };
    typedef std::map<std::string, InternalPathEnum> StringPathMap;
    typedef std::map<InternalPathEnum, std::string> PathStringMap;
    typedef std::pair<std::string, InternalPathEnum> StringPathPair;
    typedef std::pair<InternalPathEnum, std::string> PathStringPair;
    typedef std::pair<std::string, WWidget*> StringWWidgetPair;

    static StringPathPair stringPathPair[];
    static PathStringPair pathStringPair[];
    static StringPathMap  stringPathMap;
    static PathStringMap  pathStringMap;
    static bool m_bPathInitialized;

    InternalPathEnum path2Enum(const std::string& path);
    bool enum2Path(InternalPathEnum pathEnum, std::string& path);
    void SendoutActivation();
    void Initialize();
    void InitializePath();
    void InitializeMainMenu();
    void InitializeUser();
    void InitializeTask();
    void UpdateInternalPathByChoice(InternalPathEnum choice);
    bool StateMachine(const std::string& path, InternalPathEnum& result);
    void UpdateInternalPath();


};



#endif // SESSIONMANAGERWIDGET_H_INCLUDED
