#include "TaskViewWidget.h"
#include "MySqlCon.h"
#include <vector>
#include "log.h"
#include <Wt/WColor>
#include <boost/lexical_cast.hpp>

using namespace mysqlpp;

WStandardItem* TaskViewWidget::checkedItem = NULL;

TaskViewWidget::TaskViewWidget(WContainerWidget* parent)
    : WTreeView(parent), m_model(NULL), m_taskId(-1), m_width(800), m_height(600)
        ,m_notifyMethod(NULL), m_notifyObj(NULL)
{
    setAlternatingRowColors(true);
    setRowHeight(25);

    resize(WLength::Auto, WLength::Auto);

    setColumnWidth(0, WLength(120));
    setColumnAlignment(0, AlignCenter);
    setHeaderAlignment(0, AlignCenter);
    setColumnWidth(1, WLength(60));
    setColumnAlignment(1, AlignCenter);
    setHeaderAlignment(1, AlignCenter);
    setColumnWidth(2, WLength(150));
    setColumnAlignment(2, AlignCenter);
    setHeaderAlignment(2, AlignCenter);
    setColumnWidth(3, WLength(100));
    setColumnAlignment(3, AlignCenter);
    setHeaderAlignment(3, AlignCenter);
    setColumnWidth(4, WLength(60));
    setColumnAlignment(4, AlignCenter);
    setHeaderAlignment(4, AlignCenter);

    m_model = new WStandardItemModel(0, 5, this);
                        //result = new WStandardItemModel(0, 6, this);
    m_model->setHeaderData(0, Horizontal, std::string("Task"));
    m_model->setHeaderData(1, Horizontal, std::string("Status"));
    m_model->setHeaderData(2, Horizontal, std::string("StartTime"));
    m_model->setHeaderData(3, Horizontal, std::string("RunTime"));
    m_model->setHeaderData(4, Horizontal, std::string("Detail"));

    setModel(m_model);
    m_model->itemChanged().connect(this, &TaskViewWidget::itemChanged);
    //setColumnBorder(Wt::WColor(128, 255, 128));

    WApplication::instance()->useStyleSheet("icons/wt.css");
    setSelectionMode(SingleSelection);
}

void TaskViewWidget::itemChanged(WStandardItem* item)
{
    if (item->isChecked())
    {
        if (checkedItem)
        {
            checkedItem->setChecked(false);
        }
        checkedItem = item;
    }
    else
    {
        if (item == checkedItem)
        {
            checkedItem = NULL;
        }
    }
}

void TaskViewWidget::clearModel()
{
    m_model->clear();
}

bool TaskViewWidget::createModel(int nTaskId)
{
    try
    {
        MySqlCon mySqlCon;
        mysqlpp::Query query =  mySqlCon.con.query();
        //query << "select TaskName, JobName, JobStatus, JobStartTime, JobRunTime from Render_Jobs_View where TaskId="
        //    <<nTaskId;
        query<<"select t.TaskMainFile as TaskName, r.JobName as JobName, r.JobStartTime as JobStartTime, r.JobRunTime as JobRunTime "
            <<" from Tasks_View t left join Render_Jobs_View r on t.TaskId=r.TaskId where t.TaskId="<<nTaskId;
        StoreQueryResult res = query.store();
        if (res.size() > 0)
        {
            std::string fileName = res[0][0].c_str();
            size_t pos = fileName.find_last_of('/');
            if (pos != std::string::npos)
            {
                fileName = fileName.substr(pos + 1);
            }

            WStandardItem* taskItem = new WStandardItem(fileName.c_str());
            taskItem->setIcon("icons/document.png");
            m_model->appendRow(taskItem);

            for (size_t i = 0; i < res.size(); i ++)
            {
                std::vector<WStandardItem*> itemVector;
                WStandardItem* item;
                std::string jobName = res[i][1].c_str();
                pos = jobName.find_first_of(':');
                if (pos != std::string::npos)
                {
                    jobName = "frame#" + jobName.substr(0, pos);
                }
                item = new WStandardItem(jobName);
                item->setIcon("icons/frame.png");
                itemVector.push_back(item);

                item = new WStandardItem(res[i][2].c_str());
                item->setIcon("icons/status.png");
                itemVector.push_back(item);

                item = new WStandardItem(res[i][3].c_str());
                item->setIcon("icons/time.png");
                itemVector.push_back(item);


                item = new WStandardItem(res[i][4].c_str());
                item->setIcon("icons/clock.png");
                itemVector.push_back(item);

                item = new WStandardItem();
                item->setCheckable(true);
                item->setChecked(false);
                itemVector.push_back(item);
                taskItem->appendRow(itemVector);
            }
            return true;
        }
    }
    catch(Exception e)
    {
        FILE_LOG(logERROR)<<"TaskViewWidget::createModel"<<nTaskId<<"failed";
    }
    return false;
}

bool TaskViewWidget::setTaskId(int nTaskId)
{
    if (nTaskId < 1)
    {
        return false;
    }
    checkedItem = NULL;

    m_taskId = nTaskId;
    if (createModel(nTaskId))
    {
        return true;
    }
    return false;
}

void TaskViewWidget::setupNotifyEventMethod(NotifyMethod notifyMethod, WObject* notifyObj)
{
    m_notifyMethod = notifyMethod;
    m_notifyObj = notifyObj;
}
