#include "UploadFileWidget.h"
#include <vector>
#include <Wt/Http/Request>
#include <web/DomElement.h>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>


using namespace Wt::Http;

char* UploadFileWidget::ClickToStartText  = "Click select button to start uploading a file...";
char* UploadFileWidget::ClickToSelectText = "click browse button to select a file for uploading...";
char* UploadFileWidget::ClickToUploadText = "click to upload button to upload selected file now!";

UploadFileWidget::UploadFileWidget(WContainerWidget* parent)
    :WCompositeWidget(parent), m_fileUpload(NULL), m_progress(NULL)
{
    //m_panel = new WPanel();
    m_layout = new WTable();
    setImplementation(m_layout);
    m_msg = new WText();


    // Provide a button
    m_selectButton = new WPushButton("select");
    //m_uploadButton = new WPushButton("upload");
    //m_selectButton->enable();
    //m_uploadButton->disable();

    m_layout->elementAt(1, 0)->addWidget(m_msg);
    m_layout->elementAt(1, 0)->resize(this->width(), 50);
    m_msg->setText(ClickToStartText);

    //m_layout->elementAt(0, 0)->addWidget(m_uploadButton);
    m_layout->elementAt(0, 0)->addWidget(m_selectButton);
    m_selectButton->clicked().connect(this, &UploadFileWidget::initialize);
    //m_uploadButton->clicked().connect(this, &UploadFileWidget::uploadFile);
}

UploadFileWidget::~UploadFileWidget()
{
}

void UploadFileWidget::setTargetPath(const std::string& strPath)
{
    m_targetPath = strPath;
}

void UploadFileWidget::setupNotifyEventMethod(NotifyMethod notifyMethod, WObject* notifyObj)
{
    m_notifyMethod = notifyMethod;
    m_notifyObj = notifyObj;
}

void UploadFileWidget::initialize()
{
    m_selectButton->disable();
    //m_uploadButton->enable();
    if (m_progress)
    {
        m_progress->hide();
    }
    m_progress = new WProgressBar();
    m_fileUpload = new WFileUpload();
    m_fileUpload->setProgressBar(m_progress);

    std::string str = "<p>";
    str +=  ClickToSelectText;
    str += "</p>";
    m_msg->setText(str);
    m_layout->elementAt(2, 0)->addWidget(m_fileUpload);

    m_fileUpload->changed().connect(this, &UploadFileWidget::uploadFile);
    m_fileUpload->uploaded().connect(this, &UploadFileWidget::fileUploaded);

    m_fileUpload->fileTooLarge().connect(this, &UploadFileWidget::fileTooLarge);
}

void UploadFileWidget::uploadFile()
{
    if (m_fileUpload->canUpload())
    {
        m_msg->setText("preparing uploading file...");
        //m_uploadButton->disable();
        m_fileUpload->upload();
    }
}

void UploadFileWidget::fileUploaded()
{
    std::string str = "<p>upload file successful</p><p>";
    str += ClickToStartText;
    str += "</p>";

    m_msg->setText(str);

    if (m_targetPath.size() > 0)
    {
        std::string clientFile = m_fileUpload->clientFileName().toUTF8();
        size_t pos = clientFile.find_last_of('/');
        if (pos != std::string::npos)
        {
            clientFile = clientFile.substr(pos);
        }
        boost::filesystem::path dst_path(m_targetPath + "/" + clientFile);
        boost::filesystem::path src_path(m_fileUpload->spoolFileName());
        /////////////////////
        //debugging
        //str = "src:" + src_path.string() + "dst:" +  dst_path.string();
        //m_msg->setText(str);
        ///////////////////////////////
        try
        {
            boost::filesystem::copy_file(src_path, dst_path, boost::filesystem::copy_option::overwrite_if_exists);
        }
        catch (std::exception e)
        {
            str = e.what();
            m_msg->setText("error!" + str);
        }
    }
    m_selectButton->enable();
    if (m_notifyMethod)
    {
        boost::bind<void>(m_notifyMethod, m_notifyObj, _1)(UploadSuccessful);
    }
}

void UploadFileWidget::fileTooLarge(int nFileSize)
{
    m_msg->setText("upload file failed");
    if (m_notifyMethod)
    {
        boost::bind<void>(m_notifyMethod, m_notifyObj, _1)(UploadFailed);
    }
}


